/**************************************************************************
* All utility functions shared between different XUL thinggies are placed
* in this seperate file.
**************************************************************************/
const XPPREF = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);

/**************************************************************************
* A utility function that, given a folder id in bookmarks, returns a list
* of children (i.e. sub-folders and items) that the folder contains
**************************************************************************/
function getBMDSCChildren(aResource) {
    var resultArray = new Array();
    try {
	if (typeof(aResource) == "string") { aResource = RDF.GetResource(aResource); }

	RDFC.Init(BMDS, aResource);
	var containerChildren = RDFC.GetElements();

	while (containerChildren.hasMoreElements()) {
	    var res = containerChildren.getNext().QueryInterface(kRDFRSCIID);

	    if (RDFCU.IsContainer(BMDS,res)) {
		resultArray = resultArray.concat(this.getBMDSCChildren(res));
	    } else {
		resultArray.push(res);
	    }
	}
    } catch (e) { dump("getBMDSCChildren(): Error: " + e + "\n"); }

    return resultArray;
}


/**************************************************************************
* Function to read bookmark items' property. Idea from sage.mozdev.org's commonfuncs.js
**************************************************************************/
function getBMDSProperty(aResource, aArcURI) {
    // Cast into RDF
    if (typeof(aResource) == "string") { aResource = RDF.GetResource(aResource); }
    if (typeof(aArcURI) == "string") { aArcURI = RDF.GetResource(aArcURI); }

    return getBMDSTargetByURL(aResource, aArcURI).Value;
}


/**************************************************************************
* Function to get a bookmark target. Idea from sage.mozdev.org's commonfuncs.js
**************************************************************************/
function getBMDSTargetByURL(aResource, aArcURI) {
    if (typeof(aResource) == "string") { aResource = RDF.GetResource(aResource); }
    var node = BMDS.GetTarget(aResource, aArcURI, true);

    try {
	return node.QueryInterface(kRDFRSCIID);
    } catch (e) {
	try {
	    return node.QueryInterface(Components.interfaces.nsIRDFDate);
	} catch (e) {
	    return node ? node.QueryInterface(kRDFLITIID): RDF.GetLiteral("");
	}
    }
}


/**************************************************************************
* Function to create/write bookmark items' properties. Idea from sage.mozdev.org's commonfuncs.js
**************************************************************************/
function setBMDSProperty(aResource, aArcURI, aNewValue) {
    if (typeof(aResource) == "string") { aResource = RDF.GetResource(aResource); }
    if (typeof(aArcURI) == "string") { aArcURI = RDF.GetResource(aArcURI); }

    if (typeof(aNewValue) == "string") {
	aNewValue = RDF.GetResource(aNewValue);
    } else if (typeof(aNewValue) == "number") {
	aNewValue = RDF.GetIntLiteral(aNewValue);
    }

    var aOldValue = getBMDSTargetByURL(aResource, aArcURI);
    var changed = false;
    if (aArcURI && (aOldValue || aNewValue) && aOldValue != aNewValue) {
	if (aOldValue && !aNewValue) {
	    BMDS.Unassert(aResource, aArcURI, aOldValue);
	} else if (!aOldValue && aNewValue) {
	    BMDS.Assert(aResource, aArcURI, aNewValue, true);
	} else {
	    BMDS.Change(aResource, aArcURI, aOldValue, aNewValue);
	}
	changed = true;
    }
    BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
    return changed;
}

/**************************************************************************
* Function to set preferences. Idea from sage.mozdev.org's commonfuncs.js
**************************************************************************/
function setPrefValue(aPrefString, aPrefType, aValue) {
    var nsISupportsString = Components.interfaces.nsISupportsString;

    try {
	switch (aPrefType){
	    case "wstr":    // Complex string
		var string = Components.classes['@mozilla.org/supports-string;1'].createInstance(nsISupportsString);
		string.data = aValue;
		return XPPREF.setComplexValue(aPrefString, nsISupportsString, string);
		break;
	    case "str":	    // String
		return XPPREF.setCharPref(aPrefString, aValue);
		break;
	    case "int":	    // Integer
		aValue = parseInt(aValue);
		return XPPREF.setIntPref(aPrefString, aValue);
		break;
	    case "bool":    // Boolean and default
	    default:
		if(typeof(aValue) == "string"){
		    aValue = (aValue == "true");
		}
		return XPPREF.setBoolPref(aPrefString, aValue);
		break;
	}
    }catch(e){
	dump("setPrefValue: Error setting preference " + aPrefString + " to " + aValue + ":" + e + "\n");
    }
    return null;
}

/**************************************************************************
* Function to get preferences. Idea from sage.mozdev.org's commonfuncs.js
**************************************************************************/
function getPrefValue(aPrefString, aPrefType, aDefault) {
    var nsISupportsString = Components.interfaces.nsISupportsString;

    if(XPPREF.getPrefType(aPrefString) == XPPREF.PREF_INVALID) {
	dump("getPrefValue: Error getting preference " + aPrefString + ". Returning default.\n");
	return aDefault;
    }

    try {
	switch (aPrefType){
	    case "wstr":
		return XPPREF.getComplexValue(aPrefString, nsISupportsString).data;
		break;
	    case "str":
		return XPPREF.getCharPref(aPrefString).toString();
		break;
	    case "int":
		return XPPREF.getIntPref(aPrefString);
		break;
	    case "bool":
	    default:
		return XPPREF.getBoolPref(aPrefString);
		break;
	}
    }catch(e){
	dump("getPrefValue: Error getting preference " + aPrefString + ":" + e + "\n");
    }
    return aDefault;
}

/**************************************************************************
* Function to clear one preference. Idea from sage.mozdev.org's commonfuncs.js
**************************************************************************/
function clearPref(aPrefString) {
    try {
	XPPREF.clearUserPref(aPrefString);
	return true;
    } catch(e) { return false; }
}

/**************************************************************************
* Function to remove all preference. Idea from sage.mozdev.org's commonfuncs.js
**************************************************************************/
function removePrefs() {
    var prefBranch = XPPREF.getBranch("extensions.habarixenu.");
    try {
	prefBranch.deleteBranch("");
	return true;
    } catch(e) { return false; }
}

/**************************************************************************
* Function to remove convert a string from Unicode the the user's current
* encoding. Idea from sage.mozdev.org's commonfuncs.js
**************************************************************************/
function convertCharCodeFrom(aString, aCharCode) {
    var UConvID = "@mozilla.org/intl/scriptableunicodeconverter";
    var UConvIF  = Components.interfaces.nsIScriptableUnicodeConverter;
    var UConv = Components.classes[UConvID].getService(UConvIF);

    var tmpString = "";
    try{
	UConv.charset = aCharCode;
	tmpString = UConv.ConvertFromUnicode(aString);
	// UConv.finish();
    }catch(e){
	dump("convertCharCodeFrom(): Error in converting from charcode: " + e + "\n");
	tmpString = null;
    }
    return tmpString;
}


/**************************************************************************
* A utility function that, given a folder id in bookmarks, returns the
* first child that is not a container
**************************************************************************/
function getBMDSCFirstFeed(aResource) {
    var resultArray = new Array();
    try {
	if (typeof(aResource) == "string") {
	    aResource = RDF.GetResource(aResource);
	}

	RDFC.Init(BMDS, aResource);
	var containerChildren = RDFC.GetElements();
	var index = 0;

	while (containerChildren.hasMoreElements()) {
	    var res = containerChildren.getNext().QueryInterface(kRDFRSCIID);
	    index++;

	    if (RDFCU.IsContainer(BMDS,res)) {
		var n = this.getBMDSCFirstFeed(res);
		if (typeof(n) == "number") {
		    index += n;
		    return index;
		}
	    } else {
		return index;
	    }
	}
    } catch (e) { dump("getBMDSCChildren(): Error: " + e + "\n"); }
}
